import pickle
import numpy as np
from wdl.build_wdl_fp_v2 import build_wdl_fingerprint_fun
from wdl.run_utils import read_csv, save_result_to_csv, save_result_to_csv2

def predict_fp_and_pvalue(input_file,weights_file,model_file):
    """
    输出分子指纹和预测的生物活性
    :param input_file: 待预测的配体分子的SMILES分子式csv文件
    :param weights_file: 训练好的wdl网络的权重值pkl文件
    :param model_file: 训练好的RF模型pkl文件
    :return:
    """
    print("Loading data...")
    print('读取待预测文件：',input_file)
    input_smiles = read_csv(input_file)
    #print(input_smiles)
    model_params = dict(fp_length=50,
                        fp_depth=4,
                        hidden_width=20,
                        h1_size=100,
                        n_estimators=100,
                        max_features='log2',
                        L2_reg=np.exp(-2))

    def build_weight_fp_experiment(input_smiles,init_weight):
        conv_layer_sizes = [model_params['hidden_width']] * model_params['fp_depth']  # [20,20,20,20].
        conv_arch_params = {'num_hidden_features': conv_layer_sizes,
                            'fp_length': model_params['fp_length'], 'normalize': 1}
        conv_fp_func, conv_parser = build_wdl_fingerprint_fun(**conv_arch_params)
        input_fp = conv_fp_func(init_weight, input_smiles)
        return input_fp

    print("读取权重文件:",weights_file)
    with open(weights_file, 'rb') as fr:
        trained_weights = pickle.load(fr)
    fp = build_weight_fp_experiment(input_smiles,trained_weights)

    print('读取模型文件',model_file)
    with open(model_file, 'rb') as fr:
        rf_model = pickle.load(fr)
    p_value = rf_model.predict(fp)

    return fp,p_value

if __name__ == "__main__":
    input_file = 'E:\\keti_data\\A1.csv'
    temp = input_file.split('\\')[-1].split('.')[0]
    trained_weights_file = 'model\\'+temp+'_weights.pkl'
    model_file = 'model\\'+temp+'_rf.pkl'
    fp, pvalue = predict_fp_and_pvalue(input_file, trained_weights_file, model_file)
    save_result_file = 'result\\'+temp+'_pvalue_result.csv'
    save_result_to_csv(input_file, save_result_file, pvalue, 'predict_pvalue')



import autograd.numpy.random as npr
import pickle
from wdl.build_wdl_fp_v2 import build_wdl_deep_net, build_wdl_fingerprint_fun
from wdl.optimizers import adam
from wdl.util import build_batched_grad, rmse, Rs2
from sklearn.ensemble import RandomForestRegressor
from autograd import grad
from wdl.run_utils import InitParam2, divide_data, save_experiment


def train_nn(init_weights, pred_fun, loss_fun, num_weights, train_smiles, train_raw_targets, train_params,
             validation_smiles=None, validation_raw_targets=None):
    """loss_fun has inputs (weights, smiles, targets)"""
    print("Total number of weights in the network:", num_weights)
    #init_weights = npr.RandomState(seed).randn(num_weights) * train_params['init_scale']
    train_targets = train_raw_targets
    training_curve = []

    def callback(weights, iter):
        if iter % 50 == 0:
            # print ("max of weights", np.max(np.abs(weights)))
            # train_preds = undo_norm(pred_fun(weights, train_smiles[:num_print_examples]))
            train_preds = pred_fun(weights, train_smiles)
            cur_loss = loss_fun(weights, train_smiles, train_targets)
            training_curve.append(cur_loss)
            print("---------------")
            print("Iteration", iter, "loss", cur_loss)
            print("train RMSE", rmse(train_preds, train_raw_targets), \
                  "train R^2", Rs2(train_preds, train_raw_targets))
            if validation_smiles is not None:
                validation_preds = pred_fun(weights, validation_smiles)
                print("Validation RMSE", iter, ":", rmse(validation_preds, validation_raw_targets))
                print("Validation R^2", iter, ":", Rs2(validation_preds, validation_raw_targets))

    # Build gradient using autograd.
    grad_fun = grad(loss_fun)
    grad_fun_with_data = build_batched_grad(grad_fun, train_params['batch_size'],
                                            train_smiles, train_targets)

    # Optimize weights.
    trained_weights = adam(grad_fun_with_data, init_weights, callback=callback,
                           num_iters=train_params['num_iters'], step_size=train_params['step_size'])

    def predict_func(new_smiles):
        """Returns to the original units that the raw targets were in."""
        return pred_fun(trained_weights, new_smiles)

    return predict_func, trained_weights, training_curve

def train_one(param2,train_inputs,train_targets,test_inputs,test_targets):
    print("Loading data...")
    task_params = param2.task_params
    N_train, N_test = train_targets.shape,test_targets.shape
    model_params = param2.model_params
    train_params = param2.train_params
    vanilla_net_params = param2.vanilla_net_params

    def build_weight_fp_experiment(init_weight):
        conv_layer_sizes = [model_params['hidden_width']] * model_params['fp_depth']  # [20,20,20,20].
        conv_arch_params = {'num_hidden_features': conv_layer_sizes,
                            'fp_length': model_params['fp_length'], 'normalize': 1}
        conv_fp_func, conv_parser = build_wdl_fingerprint_fun(**conv_arch_params)
        train_x_fp = conv_fp_func(init_weight, train_inputs)
        test_x_fp = conv_fp_func(init_weight, test_inputs)
        return train_x_fp, test_x_fp

    def run_weight_fp_experiment():
        hidden_layer_sizes = [model_params['hidden_width']] * model_params['fp_depth']
        hidden_arch_params = {'num_hidden_features': hidden_layer_sizes,
                              'fp_length': model_params['fp_length'], 'normalize': 1}
        loss_fun, pred_fun, wfp_parser = \
            build_wdl_deep_net(hidden_arch_params, vanilla_net_params, model_params['L2_reg'])
        num_weights = len(wfp_parser)
        if len(param2.source_domain_weights_file)!=0:
            with open(param2.source_domain_weights_file,'rb') as fr:
                init_weights = pickle.load(fr,encoding='latin1')
        else:
            init_weights = npr.RandomState(0).randn(num_weights) * train_params['init_scale']
        predict_func, trained_weights, conv_training_curve = \
            train_nn(init_weights, pred_fun, loss_fun, num_weights, train_inputs, train_targets,
                     train_params, validation_smiles=test_inputs, validation_raw_targets=test_targets)
        return trained_weights

    print("Starting weight fingerprint experiment...")
    trained_weights = run_weight_fp_experiment()
    train_x, test_x = build_weight_fp_experiment(trained_weights)
    train_y = train_targets
    test_y = test_targets
    print("*******************WDL_RF2********************")
    print("input_file:", param2.task_params['data_file'])
    print("N_train:", N_train[0], "N_test:", N_test[0])
    print("num_iters:", train_params["num_iters"])
    print("batch_size:", train_params["batch_size"])
    clf = RandomForestRegressor(param2.model_params['n_estimators'])
    clf.fit(train_x,train_y)
    test_predict = clf.predict(test_x)
    Rmse = rmse(test_predict,test_y)
    R2 = Rs2(test_predict,test_y)
    print("rmse:",Rmse)
    print("R2:",R2)
    save_experiment(param2.record_file, param2.task_params['data_file'], param2.train_params['num_iters'],
                   param2.train_params['batch_size'], Rmse, R2)
    if param2.source_domain_weights_file:
        print("使用源域训练得到的权重值做迁移学习,初始化权重为：",param2.source_domain_weights_file)
    if (param2.save_params['flag']):
        print("开启模型保存")
        with open(param2.save_params['weights_file'], 'wb') as fw:
            pickle.dump(trained_weights, fw)
        with open(param2.save_params['model_file'],'wb') as fw:
            pickle.dump(clf,fw)
    print("************************************************")

def train_experiment(param2,k=1):
    """
    训练wdl-rf2模型
    :param param2: 训练模型的一些必要参数类
    :param k: 做k折交叉验证，默认k=1不做交叉验证，直接将数据集按3：1切分为训练集和测试集
    :return:
    """
    for (train_inputs,train_targets,test_inputs,test_targets) in divide_data(param2.task_params['data_file'],k):
        train_one(param2,train_inputs,train_targets,test_inputs,test_targets)

if __name__ == '__main__':
    param2 = InitParam2("E:\\keti_data\\A1.csv", 100, 44)
    param2.set_save_record_file("experiment_record.csv")
    #param2.set_save_model_file("model\\A1_weights.pkl","model\\A1_rf.pkl")
    param2.set_source_domain_weights_file("E:\\keti_data\\trained_weights1AS1.pkl")
    train_experiment(param2)



from wdl.run_utils import save_result_to_csv2, save_result_to_csv, write_csv, read_csv, InitParam, InitParam2, get_example_data

from wdl.demo_train import train_experiment
from wdl.demo_predict import predict_fp_and_pvalue

from wdl.demo_train_v2 import train_experiment as train_experiment2
from wdl.demo_predict_v2 import predict_fp_and_pvalue as predict_fp_and_pvalue2

from wdl.demo_train_tr import train_experiment_tr, train_experiment2_tr


import pickle
import numpy
from wdl.build_wdl_fp_v0 import build_wdl_fingerprint_fun
from wdl.run_utils import read_csv
from wdl.run_utils import save_result_to_csv2, save_result_to_csv


def predict_fp_and_pvalue(input_file,weights_file,model_file):
    """
    输出分子指纹和预测的生物活性
    :param input_file: 待预测的配体分子的SMILES分子式csv文件
    :param weights_file: 训练好的wdl网络的权重值pkl文件
    :param model_file: 训练好的RF模型pkl文件
    :return:
    """
    print("Loading data...")
    print('读取待预测文件：',input_file)
    input_smiles = read_csv(input_file)
    #print(input_smiles)
    model_params = dict(fp_length=50,
                        fp_depth=4,
                        hidden_width=20,
                        h1_size=100,
                        layer_weight=0.5,
                        n_estimators=100,
                        max_features='sqrt',
                        L2_reg=numpy.exp(-2))

    def build_single_weight_fp_experiment(input_smiles, init_weights, x=0):
        fp_depth = x
        hidden_layer_sizes = [model_params['hidden_width']] * fp_depth
        hidden_arch_params = {'num_hidden_features': hidden_layer_sizes,
                              'fp_length': model_params['fp_length'], 'normalize': 1}
        fp_func, conv_parser = build_wdl_fingerprint_fun(**hidden_arch_params)
        datafp = fp_func(init_weights, input_smiles)
        return datafp

    def build_weight_fp_experiment(input_smiles,init_weight):
        train_x0 = build_single_weight_fp_experiment(input_smiles,init_weight, 0)
        train_x = model_params['layer_weight'] * train_x0
        for i in range(1, model_params['fp_depth']):
            train_x1 = build_single_weight_fp_experiment(input_smiles,init_weight, i)
            train_x = train_x + model_params['layer_weight'] * (train_x1 - train_x0)
            train_x0 = train_x1
        train_xx = build_single_weight_fp_experiment(input_smiles,init_weight, model_params['fp_depth'])
        train_x = train_x + train_xx - train_x0
        return train_x

    print("读取权重文件:",weights_file)
    with open(weights_file, 'rb') as fr:
        trained_weights = pickle.load(fr)
    fp = build_weight_fp_experiment(input_smiles,trained_weights)

    print('读取模型文件',model_file)
    with open(model_file, 'rb') as fr:
        rf_model = pickle.load(fr)
    p_value = rf_model.predict(fp)

    return fp,p_value

if __name__ == "__main__":
    input_file = 'E:\\keti_data\\A1.csv'
    temp = input_file.split('\\')[-1].split('.')[0]
    trained_weights_file = 'E:\\wdl_rf_package\\wdl_rf\\model\\'+temp+'_trained_weights.pkl'
    model_file = 'E:\\wdl_rf_package\\wdl_rf\\model\\'+temp+'_RF.pkl'
    fp, pvalue = predict_fp_and_pvalue(input_file, trained_weights_file, model_file)
    save_result_file = 'E:\\wdl_rf_package\\wdl_rf\\result\\'+temp+'_result.csv'
    save_result_to_csv2(input_file, save_result_file, pvalue, fp)



import csv
import numpy as np
from wdl.util import wdl_lose_fun, rmse
from sklearn.model_selection import KFold
from os.path import dirname,join

def get_example_data(filename):
    """
    使用包中自带的数据集
    :param filename: 数据集名称
    :return: 包中数据集的真实文件路径
    """
    print("使用包中自带的数据集:",filename)
    if filename not in ['A1.csv',
                        'A2.csv',
                        'A3.csv',
                        'A4.csv',
                        'A5.csv',
                        'AS1.csv',
                        'AS2.csv']:
        raise Exception("文件不存在，样例数据集的文件名为'A1.csv','A2.csv',"
                        + "'A3.csv','A4.csv','A5.csv','AS1.csv','AS2.csv'")
    module_path = dirname(__file__)
    file_path = join(module_path,'data',filename)
    return file_path
	
def file_split(filename):
    """
    将数据集切分为训练集、验证集、测试集
    :param filename:
    :return:
    """
    with open(filename, 'r') as f:
        num = len(f.readlines()) - 1
    N_val = int(round(num / 5.0))
    N_test = int(round(num / 5.0))
    N_train = num - N_val - N_test
    return (N_train, N_val, N_test)


def file_split2(filename):
    """
    将数据集按3：1切分为训练集、测试集
    :param filename:
    :return:（训练集的个数，测试集的个数）
    """
    with open(filename, 'r') as f:
        num = len(f.readlines()) - 1
    N_test = int(round(num / 4.0))
    N_train = num - N_test
    return (N_train, N_test)


def save_experiment(filename, c1, c2, c3, c4, c5):
    """
    记录训练的结果
    :param filename: 保存的文件
    :param c1: input_File
    :param c2: num_iters
    :param c3: batch_size
    :param c4: RMSE
    :param c5: R2
    :return:
    """
    if filename:
        print("实验结果保存到：", filename)
        with open(filename, 'a', newline='') as fw:
            writer = csv.writer(fw)
            writer.writerow([c1, c2, c3, c4, c5])


def read_csv(filename):
    """从输入的文件中获取smiles分子式,输入文件的第一行是列名"""
    res = []
    with open(filename, 'r') as fr:
        reader = csv.DictReader(fr)
        for line in reader:
            res.append(line['smiles'])
    return res


def write_csv(filename, data):
    """将结果data存到filename文件中"""
    with open(filename, 'w', newline='') as fw:
        writer = csv.writer(fw)
        for i in data:
            writer.writerow([i])


def save_result_to_csv(inputfile, outputfile, data, column_name):
    """
    保存结果
    :param inputfile: 原始输入的待预测配体分子的文件，作为保存结果文件的模板
    :param outputfile: 输出结果所保存的文件
    :param data: 预测得到的结果
    :param column_name: 保存文件中当前列的列名
    :return:
    """
    print('得出', column_name, '存放在', outputfile)
    with open(inputfile, 'r') as fr:
        lines = csv.reader(fr)
        with open(outputfile, 'w', newline='') as fw:
            writer = csv.writer(fw)
            i = 0
            for l in lines:
                if i == 0:
                    l.append(column_name)
                    writer.writerow(l)
                else:
                    l.append(data[i - 1])
                    writer.writerow(l)
                i += 1


def save_result_to_csv2(inputfile, outputfile, pvalue, fp):
    """
    保存预测得到的活性和分子指纹,文件第一行都是列名
    :param inputfile: 原始输入的待预测配体分子的文件
    :param outputfile: 输出结果所保存的文件
    :param pvalue: 预测得到的配体分子与gpce的结合活性
    :param fp: 生成的分子指纹
    :return:
    """
    print('输出的分子指纹和生物活性所保存的文件：', outputfile)
    with open(inputfile, 'r') as fr:
        lines = csv.reader(fr)
        with open(outputfile, 'w', newline='') as fw:
            writer = csv.writer(fw)
            i = 0
            for l in lines:
                if i == 0:
                    l.append('predict_value')
                    l.append('fp')
                    writer.writerow(l)
                else:
                    l.extend([pvalue[i - 1], fp[i - 1]])
                    writer.writerow(l)
                i += 1


def divide_data(file, k=1):
    """
    读取file文件，按k折交叉验证拆分数据集。当k=1时不做交叉验证，直接按3：1切分训练集和测试集
    :param file:
    :param k:
    :return:
    """
    with open(file) as csv_file:
        csv_reader = csv.reader(csv_file)
        line_count = -1
        smiles, value, res = [], [], []
        for row in csv_reader:
            line_count += 1
            if line_count != 0:
                """去除第一行列名"""
                smiles.append(f'{row[2]}')
                value.append(float(f'{row[1]}'))
        X = np.asarray(smiles)
        Y = np.asarray(value)
    if k > 1:
        kf = KFold(n_splits=k, shuffle=True, random_state=1)
        # kf = KFold(n_splits=k)
        for train_index, test_index in kf.split(X, Y):
            train_x = X[train_index]
            train_y = Y[train_index]
            test_x = X[test_index]
            test_y = Y[test_index]
            res.append((train_x, train_y, test_x, test_y))
            # print('test:\n',test_index,test_index.shape)
    else:
        """不做交叉验证，直接将数据按3：1切分为训练集和验证集"""
        N_test = int(round(line_count / 4.0))
        N_train = line_count - N_test
        train_index = range(N_train)
        test_index = range(N_train, line_count)
        train_x = X[train_index]
        train_y = Y[train_index]
        test_x = X[test_index]
        test_y = Y[test_index]
        res.append((train_x, train_y, test_x, test_y))
        # print('train:\n',list(train_index))
        # print('test:\n', list(test_index))
        # print(test_y)
    return res


class InitParam():
    """wdl_rf训练所需要的参数类"""
    file_params = {'target_name': 'STANDARD_VALUE',
                   'input_file': 'E:\keti_data\A1.csv', }

    train_params = dict(num_iters=200,
                        batch_size=5,
                        init_scale=np.exp(-4),
                        step_size=0.01)

    model_params = dict(fp_length=50,
                        fp_depth=4,
                        hidden_width=20,
                        h1_size=100,
                        layer_weight=0.5,
                        n_estimators=100,
                        max_features='sqrt',
                        L2_reg=np.exp(-2))
    vanilla_net_params = dict(
        layer_sizes=[model_params['fp_length'], model_params['h1_size']],
        normalize=True, L2_reg=model_params['L2_reg'], nll_func=rmse)

    save_params = dict(flag=False, weights_file="weights.pkl", model_file="model.pkl")
    record_file = ""
    source_domain_weights_file = ""

    def __init__(self, filename, n_iters, batch_size):
        self.file_params['input_file'] = filename
        self.train_params['num_iters'] = n_iters
        self.train_params['batch_size'] = batch_size

    def set_save_model_file(self, weights_file, model_file):
        """
        设置保存模型的文件
        :param weights_file: wdl部分的权重，文件路径写全路径
        :param model_file: rf模型
        :return:
        """
        self.save_params['flag'] = True
        self.save_params['weights_file'] = weights_file
        self.save_params['model_file'] = model_file

    def set_save_record_file(self, filename):
        """
        设置训练结果的记录文件
        :param filename: 存放训练结果的文件
        :return:
        """
        self.record_file = filename

    def set_source_domain_weights_file(self, filename):
        """
        运用迁移学习，设置源域训练得到的权重文件
        :param filename:
        :return:
        """
        self.source_domain_weights_file = filename


class InitParam2():
    """wdl_rf2训练模型的一些必要参数类"""
    task_params = {'target_name': 'STANDARD_VALUE',
                   'data_file': ''}

    model_params = dict(fp_length=50,
                        fp_depth=4,
                        hidden_width=20,
                        h1_size=100,
                        n_estimators=100,
                        max_features='log2',
                        L2_reg=np.exp(-2))

    train_params = dict(num_iters=500,
                        batch_size=30,
                        init_scale=np.exp(-4),
                        step_size=0.01)

    # Define the architecture of the network that sits on top of the fingerprints.
    vanilla_net_params = dict(
        layer_sizes=[model_params['fp_length'], model_params['h1_size']],
        normalize=True, L2_reg=model_params['L2_reg'], nll_func=wdl_lose_fun)

    save_params = dict(flag=False, weights_file="", model_file="")
    record_file = ""
    source_domain_weights_file = ""

    def __init__(self, filename, n_iters, batch_size):
        self.task_params['data_file'] = filename
        self.train_params['num_iters'] = n_iters
        self.train_params['batch_size'] = batch_size

    def set_save_model_file(self, weights_file, model_file):
        """
        设置保存模型的文件
        :param weights_file: wdl部分的权重，文件路径写全路径
        :param model_file: rf模型
        :return:
        """
        self.save_params['flag'] = True
        self.save_params['weights_file'] = weights_file
        self.save_params['model_file'] = model_file

    def set_save_record_file(self, filename):
        """
        设置训练结果的记录文件
        :param filename: 存放训练结果的文件
        :return:
        """
        self.record_file = filename

    def set_source_domain_weights_file(self, filename):
        """
        运用迁移学习，设置源域训练得到的权重文件
        :param filename:
        :return:
        """
        self.source_domain_weights_file = filename


if __name__ == "__main__":
    print(InitParam.file_params['input_file'].split('\\')[-1].split('.')[0] + '.pkl')




from wdl.demo_train import train_experiment
from wdl.run_utils import InitParam,InitParam2
from wdl.demo_train_v2 import train_experiment as train_experiment2

def train_experiment_tr(source_weights,param,k=1):
    """
    wdl_rf的迁移学习版本
    :param source_weights: 源域训练得到的权重值文件
    :param param: 必要参数
    :param k: k折交叉验证
    :return:
    """
    param.set_source_domain_weights_file(source_weights)
    train_experiment(param=param,k=k)

def train_experiment2_tr(source_weights,param,k=1):
    """
    wdl_rf2的迁移学习版本
    :param source_weights: 源域训练得到的权重文件
    :param param: 必要参数
    :param k: k折交叉验证
    :return:
    """
    param.set_source_domain_weights_file(source_weights)
    train_experiment2(param2=param,k=k)


# -*- coding: utf-8 -*-
import autograd.numpy as np
from autograd.scipy.special import logsumexp

from wdl.features import num_atom_features, num_bond_features
from wdl.util import memoize, WeightsParser
from wdl.mol_graph import graph_from_smiles_tuple, degrees
from wdl.build_wdl_net_v2 import build_fingerprint_deep_net, relu, batch_normalize,sigmoid

def fast_array_from_list(xs):
    return np.concatenate([np.expand_dims(x, axis=0) for x in xs], axis=0)

def sum_and_stack(features, idxs_list_of_lists):
    return fast_array_from_list([np.sum(features[idx_list], axis=0) for idx_list in idxs_list_of_lists])

def softmax(X, axis=0):
    return np.exp(X - logsumexp(X, axis=axis, keepdims=True))

def matmult_neighbors(array_rep, atom_features, bond_features, neighbour_weight, neighbourbond_weight):
    neighbour_activations_by_degree = []
    neighbourbond_activations_by_degree1 = []
    for degree in degrees:
        atom_neighbors_list = array_rep[('atom_neighbors', degree)]
        bond_neighbors_list = array_rep[('bond_neighbors', degree)]
        #分别生成近邻原子信息和近邻键的信息
        if len(atom_neighbors_list) > 0:
            neighbor_features = atom_features[atom_neighbors_list]
            neighborbond_features=bond_features[bond_neighbors_list]
            # dims of stacked_neighbors are [atoms, neighbors, atom and bond features]
            mean_neighbors = np.mean(neighbor_features, axis=1)
            neighbouractivations = np.dot(mean_neighbors, neighbour_weight)
            mean_neighborsbond = np.mean(neighborbond_features, axis=1)
            neighbourbondactivations = np.dot(mean_neighborsbond, neighbourbond_weight)
            neighbourbond_activations_by_degree1.append(neighbourbondactivations)
            neighbour_activations_by_degree.append(neighbouractivations)
    # This operation relies on atoms being sorted by degree,
    # in Node.graph_from_smiles_tuple()
    return np.concatenate(neighbour_activations_by_degree, axis=0),np.concatenate(neighbourbond_activations_by_degree1, axis=0)

def weights_name(layer, degree):
    return "layer " + str(layer) + " degree " + str(degree) + " filter"

def build_wdl_fingerprint_fun(num_hidden_features=[100, 100], fp_length=512,
                                  normalize=True, activation_function=relu,
                                  return_atom_activations=False):
    """Sets up functions to compute convnets over all molecules in a minibatch together."""

    # Specify weight shapes.
    parser = WeightsParser()
    all_layer_sizes = [num_atom_features()] + num_hidden_features
    parser.add_weights('fp weights', (fp_length, fp_length))
    parser.add_weights('fp bias',    (1, fp_length))
    for layer in range(len(all_layer_sizes)):
        parser.add_weights(('layer output weights', layer), (all_layer_sizes[layer], fp_length))
        parser.add_weights(('layer output bias', layer),    (1, fp_length))

    in_and_out_sizes = zip(all_layer_sizes[:-1], all_layer_sizes[1:])
    for layer, (N_prev, N_cur) in enumerate(in_and_out_sizes):
        parser.add_weights(("layer", layer, "biases"), (1, N_cur))
        parser.add_weights(("layer", layer, "self filter"), (N_prev, N_cur))
        parser.add_weights(("layer", layer, "neighbour filter"), (N_prev, N_cur))
        #加了bond权重
        parser.add_weights(("layer", layer, "neighbourbond filter"), (6, N_cur))
# =============================================================================
#         for degree in degrees:
#             parser.add_weights(weights_name(layer, degree), (N_prev + num_bond_features(), N_cur))
# =============================================================================

    def update_layer(weights, layer, atom_features, bond_features, array_rep, normalize=False):
# =============================================================================
#         def get_weights_func(degree):
#             return parser.get(weights, weights_name(layer, degree))
# =============================================================================
        layer_bias         = parser.get(weights, ("layer", layer, "biases"))
        layer_self_weights = parser.get(weights, ("layer", layer, "self filter"))
        layer_neighbour_weight=parser.get(weights,("layer", layer, "neighbour filter"))
        layer_neighbourbond_weight=parser.get(weights,("layer", layer, "neighbourbond filter"))
        self_activations = np.dot(atom_features, layer_self_weights)
        neighbour_activations,neighbourbond_activations= matmult_neighbors(
            array_rep, atom_features, bond_features, layer_neighbour_weight,layer_neighbourbond_weight)

        total_activations = neighbour_activations + self_activations + neighbourbond_activations + layer_bias
        if normalize:
            total_activations = batch_normalize(total_activations)
        return activation_function(total_activations)

    def output_layer_fun_and_atom_activations(weights, smiles):
        """Computes layer-wise convolution, and returns a fixed-size output."""

        array_rep = array_rep_from_smiles(tuple(smiles))
        atom_features = array_rep['atom_features']
        bond_features = array_rep['bond_features']

        all_layer_fps = []
        atom_activations = []
        def write_to_fingerprint(atom_features, layer):
            cur_out_weights = parser.get(weights, ('layer output weights', layer))
            cur_out_bias    = parser.get(weights, ('layer output bias', layer))
            atom_outputs = softmax(cur_out_bias + np.dot(atom_features, cur_out_weights), axis=1)
            atom_activations.append(atom_outputs)
            # Sum over all atoms within a moleclue:
            layer_output = sum_and_stack(atom_outputs, array_rep['atom_list'])
            all_layer_fps.append(layer_output)

        num_layers = len(num_hidden_features)
        for layer in range(num_layers):
            write_to_fingerprint(atom_features, layer)
            atom_features = update_layer(weights, layer, atom_features, bond_features, array_rep,
                                         normalize=normalize)
        write_to_fingerprint(atom_features, num_layers)
        fp_weights = parser.get(weights, 'fp weights')
        fp_bias    = parser.get(weights, 'fp bias')
        sum_fps=[]
        for i in range(len(all_layer_fps)):
            fp_outputs = relu(fp_bias + np.dot(all_layer_fps[i], fp_weights))
            sum_fps.append(fp_outputs)
                                    
        return sum(sum_fps), atom_activations, array_rep

    def output_layer_fun(weights, smiles):
        output, _, _ = output_layer_fun_and_atom_activations(weights, smiles)
        return output

    def compute_atom_activations(weights, smiles):
        _, atom_activations, array_rep = output_layer_fun_and_atom_activations(weights, smiles)
        return atom_activations, array_rep

    if return_atom_activations:
        return output_layer_fun, parser, compute_atom_activations
    else:
        return output_layer_fun, parser

@memoize
def array_rep_from_smiles(smiles):
    """Precompute everything we need from MolGraph so that we can free the memory asap."""
    molgraph = graph_from_smiles_tuple(smiles)
    arrayrep = {'atom_features' : molgraph.feature_array('atom'),
                'bond_features' : molgraph.feature_array('bond'),
                'atom_list'     : molgraph.neighbor_list('molecule', 'atom'), # List of lists.
                'rdkit_ix'      : molgraph.rdkit_ix_array()}  # For plotting only.
    for degree in degrees:
        arrayrep[('atom_neighbors', degree)] = \
            np.array(molgraph.neighbor_list(('atom', degree), 'atom'), dtype=int)
        arrayrep[('bond_neighbors', degree)] = \
            np.array(molgraph.neighbor_list(('atom', degree), 'bond'), dtype=int)
    return arrayrep

def build_wdl_deep_net(conv_params, net_params, fp_l2_penalty=0.0):
    """Returns loss_fun(all_weights, z, targets), pred_fun, combined_parser."""
    conv_fp_func, conv_parser = build_wdl_fingerprint_fun(**conv_params)
    return build_fingerprint_deep_net(net_params, conv_fp_func, conv_parser, fp_l2_penalty)





# -*- coding: utf-8 -*-
import autograd.numpy as np

from wdl.util import memoize, WeightsParser,wdl_lose_fun
from wdl.rdkit_utils import smiles_to_fps,smiles_to_fps1
#from rdkit.Chem import MACCSkeys
#from pychem.pychem import PyChem2d
import rdkit.Chem as Chem


def batch_normalize(activations):
    mbmean = np.mean(activations, axis=0, keepdims=True)
    return (activations - mbmean) / (np.std(activations, axis=0, keepdims=True) + 1)

#def relu(X):
#    out = X * (X > 0)               
#    return out, X
def relu(X):
    return X * (X > 0)

#    "Rectified linear activation function."


def sigmoid(x):
    return 0.5*(np.tanh(x) + 1)
    
def mean_squared_error(predictions, targets):
    return np.mean((predictions - targets)**2, axis=0)

def categorical_nll(predictions, targets):
    return -np.mean(predictions * targets)

def binary_classification_nll(predictions, targets):
    """Predictions is a real number, whose sigmoid is the probability that
     the target is 1."""
    pred_probs = sigmoid(predictions)
    label_probabilities = pred_probs * targets + (1 - pred_probs) * (1 - targets)
    return -np.mean(np.log(label_probabilities))

def build_standard_net(layer_sizes, normalize, L2_reg, L1_reg=0.0, activation_function=relu,
                       nll_func=wdl_lose_fun):
    """Just a plain old neural net, nothing to do with molecules.
    layer sizes includes the input size."""
    layer_sizes = layer_sizes + [1]

    parser = WeightsParser()
    for i, shape in enumerate(zip(layer_sizes[:-1], layer_sizes[1:])):
        parser.add_weights(('weights', i), shape)
        parser.add_weights(('biases', i), (1, shape[1]))

    def predictions(W_vect, X):
        cur_units = X
        for layer in range(len(layer_sizes) - 1):
            cur_W = parser.get(W_vect, ('weights', layer))
            cur_B = parser.get(W_vect, ('biases', layer))
            cur_units = np.dot(cur_units, cur_W) + cur_B
            if layer < len(layer_sizes) - 2:
                if normalize:
                    cur_units = batch_normalize(cur_units)
                cur_units = activation_function(cur_units)
        return cur_units[:, 0]

    def loss(w, X, targets):
        assert len(w) > 0
        log_prior = -L2_reg * np.dot(w, w) /(2*len(w)+0.0000001) - L1_reg * np.mean(np.abs(w))
        preds = predictions(w, X)
        return nll_func(preds, targets) - log_prior

    return loss, predictions, parser


def build_fingerprint_deep_net(net_params, fingerprint_func, fp_parser, fp_l2_penalty):
    """Composes a fingerprint function with signature (smiles, weights, params)
     with a fully-connected neural network."""
    net_loss_fun, net_pred_fun, net_parser = build_standard_net(**net_params)

    combined_parser = WeightsParser()
    combined_parser.add_weights('fingerprint weights', (len(fp_parser),))
    combined_parser.add_weights('net weights', (len(net_parser),))

    def unpack_weights(weights):
        fingerprint_weights = combined_parser.get(weights, 'fingerprint weights')
        net_weights         = combined_parser.get(weights, 'net weights')
        return fingerprint_weights, net_weights

    def loss_fun(weights, smiles, targets):
        fingerprint_weights, net_weights = unpack_weights(weights)
        fingerprints = fingerprint_func(fingerprint_weights, smiles)
        net_loss = net_loss_fun(net_weights, fingerprints, targets)
        if len(fingerprint_weights) > 0 and fp_l2_penalty > 0:
            return net_loss + fp_l2_penalty * np.mean(fingerprint_weights**2)
        else:
            return net_loss

    def pred_fun(weights, smiles):
        fingerprint_weights, net_weights = unpack_weights(weights)
        fingerprints = fingerprint_func(fingerprint_weights, smiles)
        return net_pred_fun(net_weights, fingerprints)

    return loss_fun, pred_fun, combined_parser




import setuptools
from os import path as os_path

with open("README.md", "r") as fh:
    long_description = fh.read()
	
this_directory = os_path.abspath(os_path.dirname(__file__))
def read_file(filename):
    with open(os_path.join(this_directory, filename), encoding='utf-8') as f:
        long_description = f.read()
    return long_description
	
def read_requirements(filename):
    return [line.strip() for line in read_file(filename).splitlines()
            if not line.startswith('#')]
setuptools.setup(
    name="wdl_rf", # Replace with your own username
    version="0.0.10",
    author="xhj,zqm",
    description="Create molecular fingerprint features of Ligands and predicting Bioactivities Acting with G Protein-coupled Receptors",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
	zip_safe=False,
	include_package_data = True,
	install_requires = read_requirements('requirements.txt'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)